/* This program calculates and displays depreciation tables
   using various methods of computing the depreciation.

   Input:  User-selected options, amounts to be depreciated,
           and number of years
   Output: User prompts, menu of options, and depreciation tables
--------------------------------------------------------------------*/

#include <iostream.h>

#include "Deprec.h"             // module of depreciation functions

void DisplayMenu(void);         // function to display the menu
char GetMenuChoice(void);       // function to input user choice

int main(void)
{
   cout << "\nThis program computes depreciation tables,\n"
        << "\tusing various methods of depreciation.\n";

   char
      Option;                   // menu option selected by user
   int
      NumYears = 0;             // number of years
   double
      Amount = 0.0;             // amount to be depreciated

   DisplayMenu();               // show the menu at the outset

   do
   {
      Option = GetMenuChoice();  // get user's choice

      switch (Option)         // perform the option selected
      {
         case 'A':            // get new Amount & NumYears
            cout << "\nHow much money is to be depreciated? ";
            cin >> Amount;
            cout << "Over how many years? ";
            cin >> NumYears;
            break;
         case 'B':            // do straight-line method
            StraightLine(Amount, NumYears);
            break;
         case 'C':            // do sum-of-years-digits method
            SumOfYears(Amount, NumYears);
            break;
         case 'D':            // redisplay the menu
            DisplayMenu();
            break;
	 case 'Q':            // do nothing -- stop processing
            break;
         default:             // execution should never get here
            cerr << "*** Invalid menu choice: " << Option << ".\n";
            DisplayMenu();
      }
   }
   while (Option != 'Q');

   return 0;
}

/*------------------------------------------------------------------
DisplayMenu outputs the menu of operations supported by the program.

   Output: The menu of options
-------------------------------------------------------------------*/

void DisplayMenu(void)
{
   cout << "\nPlease enter:\n"
        << "\tA - to enter a new Amount and/or Number of Years,\n"
        << "\tB - to use the straight-line method,\n"
        << "\tC - to use the sum-of-years-digits method,\n"
        << "\tD - to re-display this menu, or\n"
        << "\tQ - to quit.\n";
}

/*-----------------------------------------------------------------
GetMenuChoice reads and returns a menu choice.

   Output:  prompts for input.
   Input:   the user's (char) Choice.
   Return:  that choice.
------------------------------------------------------------------*/

#include <ctype.h>              // provides islower(), toupper()

char GetMenuChoice(void)
{
   char
      Choice;                   // the user\'s menu choice

   cout << "\nOption? ";        // prompt the user
   cin >> Choice;               // input their choice
   if (islower(Choice))         // if choice is lowercase
     Choice = toupper(Choice);  //   convert it to uppercase

   return Choice;
}
