/* This program reformats the data from the file "emp.dat" and
   writes it to the file "newemp.dat" in an easier-to-read format.

   Input (emp.dat):     A sequence of lines of the form:

                           employee-id-number      hourly-rate

   Output (newemp.dat): Column headings, and a sequence of lines:

                           employee-id-number      hourly-rate

                        in an easy-to-read format, followed by
                        the average hourly rate
------------------------------------------------------------------*/

#include <iostream.h>                       // for terminal I/O
#include <fstream.h>                        // for file I/O
#include <iomanip.h>                        // for formatted I/O
#include <stdlib.h>                         // for exit()

int main(void)
{
   const char
      InFile[] = "emp.dat",                 // name of input file
      OutFile[] = "newemp.dat",             // name of output file
                                            // error messages:
      OpenFailedMsg[] = "\n*** main: unable to open file: ",
      NoValuesMsg[] = "\n*** main: no values to process!\n";

   cout << "\nThis program reformats the data from the file '"
        << InFile << "'\n\tand writes it to the file '"
        << OutFile << "'.\n\n";

   fstream
      InStream(InFile, ios::in),            // stream to input file
      OutStream(OutFile, ios::out);         // stream to output file

   if (InStream.fail())                     // check for
   {                                        //   successful opens
      cerr << OpenFailedMsg << InFile << endl;
      exit (-1);
   }
   if (OutStream.fail())
   {
      cerr << OpenFailedMsg << OutFile << endl;
      exit (-1);
   }

   long
      Count = 0,                            // number of employees
      EmpIdNumber;                          // an ID number
   double
      HourlyRate,                           // hourly rate
      RateSum = 0.0;                        // total of hourly rates

                                            // write column headings
   OutStream << "\n\t" << "  Employee  " << '\t' << " Hourly Rate "
             << "\n\t" << "============" << '\t' << "============="
             << endl;

   OutStream << setprecision(2)             // 2 dec. places
             << setiosflags(ios::showpoint | ios::fixed);
                                            // fixed, not scientific

   InStream >> EmpIdNumber >> HourlyRate;   // get first line
                                            //   from the file
   while (!InStream.eof())                  // while more values
   {
      Count++;                              //   increment Count
      RateSum += HourlyRate;                //   keep running total
                                            //   reformat the lines
      OutStream << "\t "
                << EmpIdNumber / 1000000 << '-'
                << EmpIdNumber % 1000000 / 10000 << '-'
                << EmpIdNumber % 10000
                << '\t'
                << setw(9) << HourlyRate
                << endl;
                                            //   get next line
      InStream >> EmpIdNumber >> HourlyRate;//     from the file
   }                                        // end while

   InStream.close();                        // close the input file

                                            // output summary info
   if (Count > 0)
      OutStream << "\n\nThere are " << Count
                << " employees, averaging $" << RateSum / Count
                << " per hour.\n\n";
   else
      cerr << NoValuesMsg;

   OutStream.close();                       // close the output file
                                                
  cout << "\nDone. Results are in the file" //   and tell user where
       <<  " '" << OutFile << "'.\n\n";     //   to find the results

  return 0;
}
