/* This function finds the minimum of an arbitrary sequence of values.

   Receive: TypeString, an encoding of the argument types
            an arbitrary sequence of variables
   Return:  The (double) smallest of all of the values passed
------------------------------------------------------------------*/

#include <stdarg.h>
#include <float.h>               // contains DBL_MAX

#include "Strings.h"

double Minimum(const Strings& TypeString ...)
{
   va_list
      ArgList;

   va_start(ArgList, TypeString);// ArgList begins after TypeString

   double
      CurrentMin = DBL_MAX,      // initially, the largest double
      CurrentArg;                // stores value being processed

   int
      NumArgs = TypeString.Length();


   for (int i = 0; i < NumArgs; i++)
   {
      switch(TypeString[i])
      {
         case 'c': case 's':     // char, short come through as ints
         case 'i':
            CurrentArg = double( va_arg(ArgList, int) );
            break;

         case 'l':               // in case sizeof int < sizeof long
            CurrentArg = double( va_arg(ArgList, long) );
            break;

         case 'f': case 'd':     // float comes through as double
         case 'r':               // allow 'r' for any real value
            CurrentArg = double( va_arg(ArgList, double) );
            break;

         default:
            cerr << "\n*** Minimum: unknown type indicator: "
                 << TypeString[i] << endl;
      }

      if (CurrentArg < CurrentMin)// smaller number found
         CurrentMin = CurrentArg;
   }

   va_end(ArgList);

   return CurrentMin;
}
