/* This program inputs student numbers, class codes, and test
   scores.  A student number can then be input and the 
   corresponding class code and test score are retrieved.

   Input (file):     A list of Student records
   Input (keyboard): One or more student ID numbers
   Output:           The student data corresponding to the 
                        input ID number
------------------------------------------------------------------*/

#include <iostream.h>
#include <stdlib.h>

#include "List.h"                        // list of Student records
#include "Strings.h"
#include "Student.h"

int main(void)
{
  const char
     EOFMark[] = "Ctrl-D",
     InputFile[] = "Registrar.Dat";

  cout 
    << "\nThis program simulates an information retrieval system"
    << "\n\tby reading a series of student records from"
    << "'" << InputFile << "'"
    << "\n\tand then allowing retrieval of any student's data.\n";

  fstream
     InStream(InputFile, ios::in);    // open the input file

  if (InStream.fail())                      // check for failure
  {
     cerr << "\n*** Main: unable to open '"
          << InputFile << "'!\n";
     exit (-1);
  }

  int
     NumValues;

  InStream >> NumValues;

  List
     StudentList(NumValues);                // list of Student records

  InStream >> StudentList;                  // fill the list

  StudentList.SSort();                      // sort the list (by name)

  long
     Pos;                                   // index of ID in List
  Strings
     FirstName,                             // the names of the
     LastName;                              //   student being sought

  cout <<"\nEnter a student's name ("       // prompt for input
       << EOFMark << " to quit): ";
  cin >> FirstName >> LastName;             // get first name
  while (!cin.eof())                        // while more data:
  {
     Pos = StudentList.BSearch(             //   binary search for
             Student(FirstName, LastName)); //     this student

     if (Pos >= 0)                          //   if search successful
        cout << endl                        //     display the data
             << StudentList[Pos] << endl;
     else                                   //   otherwise, tell user
        cerr << "\nThere is no student whose name is "
             << FirstName << ' ' << LastName << ".\n";
     cout <<"\nEnter a student's name ("    //   get next name
          << EOFMark << " to quit): ";
     cin >> FirstName >> LastName;
  }

  return 0;
}
