/* This program provides a monthly sales report, given a sales table.

   Input (keyboard): Name of file containing sales table
   Input (file):     The dimensions and entries of the sales table
   Output (screen):  User prompts and a monthly sales report

------------------------------------------------------------------*/

#include <iostream.h>
#include <iomanip.h>

#include "Table.h"
#include "Strings.h"

void DisplayReport(const Table& Tab);

int main(void)
{                                       // explain the program
   cout << "\nThis program generates a monthly sales report,\n"
        << "\tgiven a file of the month's sales figures,\n"
        << "\torganized as a table whose rows are the auto models\n"
        << "\tand whose columns are the salespersons.\n";

                                        // get the input file name
   cout << "\nPlease enter the name of the input file: ";
   Strings
      FileName;
   cin >> FileName;

   Table                                // declare a Table
      SalesTable;

   SalesTable.Load(FileName);           // fill it from the input file

   DisplayReport(SalesTable);           // display the report

  return 0;
}


/*----- This function displays a Table as a summary report.

Receive:          A Table named Tab
Output (screen):  Tab, in easy-to-read format,
                     with column and row headings, and
                     with column and row summaries
------------------------------------------------------------------*/

void DisplayReport(const Table& Tab)
{
  const int                                        // column width
     Width = 5;

  cout << setprecision(0);                         // integer display

                                                   // column headings
  cout << "\n                          Salesperson\n";
  cout << " Models :    1    2    3    4    5    6    7    8   : Totals\n";
  cout << "------------------------------------------------------------\n";


  for (int row = 0; row < Tab.Rows(); row++)      // for each row:
  {
    cout << setw(Width) << (row + 1) << "   : ";  //  label the row

                                                  //  for each column:
    for (int col = 0; col < Tab.Columns(); col++)
      cout << setw(Width)                         //    display
           << Tab.GetElem(row, col);              //     Grid[row,col]

    cout << "   :" << setw(Width)
         << Tab.RowSum(row) << endl;              //   summarize row
  }

  cout << "---------------------------------------"
       << "-----------------------\n";
  cout << " Totals : ";                            // label Totals row

	for (int col = 0; col < Tab.Columns(); col++)    // for each column:
		cout << setw(Width) << Tab.ColumnSum(col);     //   summarize col.

  cout << "\n\n";
}
