/* This program maintains a list of Employee records.

   Input(EmpFile):  A list of employee records
   Input(keyboard): Modifications, insertions, and deletions to 
                       the list
   Output(EmpFile): The updated list
------------------------------------------------------------------*/

#include <iostream.h>

#include "LinkedList.h"   // ListElement = Employee

void Update(LinkedList& L);

int main(void)
{
   LinkedList
      L;                  // a linked list of Employee objects

   L.Load("EmpFile");     // fill it with data from EmpFile

   Update(L);             // allow user to update the list

   L.Write("EmpFile");    // write the updated list back to EmpFile

   return 0;
}


/* This function updates the linked employee list.

   Receive: L, a linked list of employees
   Input:   Modifications, insertions, and/or deletions for the list
   Return:  L, updated as specified
--------------------------------------------------------------------*/

void AddEmployee(LinkedList& L);   // add a new employee to the list

void DeleteEmployee(LinkedList& L);// delete an employee from the list

void ModifyEmployee(LinkedList& L);// modify an employee's record

void Update(LinkedList& L)
{
   const Strings
      Menu = Strings("\nPlease enter:\n") &
                "\tA - to add a new employee to the list\n" &
                "\tD - to delete an employee\n" &
                "\tM - to modify an employee's record\n" &
                "\tQ - to quit.\n--> ";
   char
      Choice;                     // the user's menu choice

   do                             // Loop1: perform multiple updates
   {
      do                          //   Loop2: get a valid menu choice
      {
         cout << Menu;
         cin >> Choice;

      }
      while ((Choice != 'A') && (Choice != 'D') &&
            ( Choice != 'M') && (Choice != 'Q'));

      switch (Choice)            //     perform their choice
      {
         case 'A':   AddEmployee(L);
                     break;
         case 'D':   DeleteEmployee(L);
                     break;
         case 'M':   ModifyEmployee(L);
                     break;
         case 'Q':   // do nothing
                     break;
         default:    cerr << "\n*** " << Choice
                          << " is not a valid option!\n";
      }
   }
   while (Choice != 'Q');        // until 'quit' is chosen
}


/* This function adds a new employee to the linked list.

   Receive: L, the linked list of employees
   Output:  Prompts for input
   Input:   An employee's data
   Return:  L, with the new employee record appended
------------------------------------------------------------------*/

void AddEmployee(LinkedList& L)
{
   int
      EmpNumber,
      EmpAge,
      EmpDependents;
   double
      EmpWage;
                                         // get the new emp's data
   cout << "\nEnter new employee's ...\n";
   cout << "\tNumber: ";
   cin >> EmpNumber;
   cout << "\tAge: ";
   cin >> EmpAge;
   cout << "\tNumber of Dependents: ";
   cin >> EmpDependents;
   cout << "\tand Wage: ";
   cin >> EmpWage;

   Employee                              // build an Employee object
      Emp(EmpNumber, EmpAge, EmpDependents, EmpWage);

   if (L.LSearch(Emp) >= 0)
      cout << "\nAdd: this employee is already in the list!\n";
   else
      L.Insert(Emp, LinkedList::End);   // append it to L
}

/* This function removes an employee from the linked list.

   Receive: L, the linked list of employees
   Output:  Prompts for input
   Input:   An employee's number
   Return:  L, with the employee's record removed
------------------------------------------------------------------*/

void DeleteEmployee(LinkedList& L)
{
   int
      EmpNumber;
                                   // get their number
   cout << "\nNumber of employee to be deleted? ";
   cin >> EmpNumber;

   Employee                        // build an Employee object
      Emp(EmpNumber);

   int                             // search for them
      Pos = L.LSearch(Emp);

   if (Pos >= 0)                   // if they're found
      L.Delete(Pos);               //   remove them
   else
      cout << "\n*** DeleteEmployee: no employee found with "
           << EmpNumber << " as their number!\n";
}

/* This function modifies an employee's record in the linked list.

   Receive: L, the linked list of employees
   Output:  Prompts for input
   Input:   The updates to the data
   Return:  L, with the employee's record updated
------------------------------------------------------------------*/

void ModifyEmployee(LinkedList& L)
{
   int
      EmpNumber,
      EmpAge,
      EmpDependents;
   double
      EmpWage;
                                   // get their number
   cout << "\nNumber of employee whose data is to be modified? ";
   cin >> EmpNumber;

   Employee                        // build an Employee object
      Emp(EmpNumber);

  int                              // search the list for them
    Pos = L.LSearch(Emp);

  if (Pos >= 0)                    // if they're found
    {                              //   do modification
      cout << "\nCurrent Data:   " << L[Pos] << endl;
      cout << "Enter new Data: ";
      cin >> EmpNumber >> EmpAge >> EmpDependents >> EmpWage;

      L[Pos] = Employee(EmpNumber, EmpAge, EmpDependents, EmpWage);
    }
  else
    cout << "\n*** Sorry - no employee with number "
         << EmpNumber << " is listed.\n";
}
